
#property indicator_separate_window
#property indicator_buffers    5
#property indicator_color1     LimeGreen
#property indicator_color2     LimeGreen
#property indicator_color3     Red
#property indicator_color4     Red
#property indicator_color5     DarkGray
#property indicator_width1     2
#property indicator_width3     2
#property indicator_width5     2

//
//
//
//
//

extern int    CCIPeriod       = 14;
extern int    CCIPrice        = PRICE_TYPICAL;
extern int    T3Period        = 5;
extern int    T3Price         = PRICE_CLOSE;
extern double T3Hot           = 0.7;
extern bool   T3Original      = true;
extern bool   ShowHistogram   = true;
extern double OverSold        = -150;
extern double OverBought      = 150;
           
extern string note            = "turn on Alert = true; turn off = false";
extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = true;
extern bool   alertsEmail     = false;
extern string soundfile       = "alert2.wav";







double cci[];
double cciU[];
double cciUd[];
double cciD[];
double cciDu[];
double prices[];
double trend[];
double slope[];
double emas[][6];

double alpha;
double c1;
double c2;
double c3;
double c4;

//
//
//
//
//

int init()
{
      IndicatorBuffers(8);
      SetIndexBuffer(0,cciU);  
      SetIndexBuffer(1,cciUd);  
      SetIndexBuffer(2,cciD);  
      SetIndexBuffer(3,cciDu);  
      SetIndexBuffer(4,cci);  
      SetIndexBuffer(5,prices);
      SetIndexBuffer(6,trend); 
      SetIndexBuffer(7,slope); 
      if (ShowHistogram)
      {
        SetIndexStyle(0,DRAW_HISTOGRAM);
        SetIndexStyle(1,DRAW_HISTOGRAM); 
        SetIndexStyle(2,DRAW_HISTOGRAM); 
        SetIndexStyle(3,DRAW_HISTOGRAM); 
      }
      else
      {
        SetIndexStyle(0,DRAW_NONE); SetIndexLabel(0,NULL);
        SetIndexStyle(1,DRAW_NONE); SetIndexLabel(1,NULL);
        SetIndexStyle(2,DRAW_NONE); SetIndexLabel(2,NULL);
        SetIndexStyle(3,DRAW_NONE); SetIndexLabel(3,NULL);
      }
      
      
      SetLevelValue(0,OverBought);
      SetLevelValue(1,OverSold); 
      SetLevelValue(2,0); 
      
      //
      //
      //
      //
      //
      
      double a = T3Hot;
            c1 = -a*a*a;
            c2 =  3*(a*a+a*a*a);
            c3 = -3*(2*a*a+a+a*a*a);
            c4 = 1+3*a+a*a*a+3*a*a;

      T3Period = MathMax(1,T3Period);
      if (T3Original)
           alpha = 2.0/(1.0 + T3Period);
      else alpha = 2.0/(2.0 + (T3Period-1.0)/2.0);
      
     IndicatorShortName("Cci T3 Smoothed(" +CCIPeriod+ ", " +T3Period+")");
   return(0);
}

//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,k,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (ArrayRange(emas,0) != Bars) ArrayResize(emas,Bars);

   //
   //
   //
   //
   //

   for(i=limit; i>=0; i--)
   {
      prices[i]  = iMA(NULL,0,1,0,MODE_SMA,CCIPrice,i);
      double avg = 0; for(k=0; k<CCIPeriod; k++) avg +=         prices[i+k];      avg /= CCIPeriod;
      double dev = 0; for(k=0; k<CCIPeriod; k++) dev += MathAbs(prices[i+k]-avg); dev /= CCIPeriod;
         if (dev!=0)
               cci[i] = iT3((prices[i]-avg)/(0.015*dev),i);
         else  cci[i] = iT3(0,                          i); 
         
         if (ShowHistogram)
         {
           cciU[i]  = EMPTY_VALUE;
           cciD[i]  = EMPTY_VALUE;
           cciUd[i] = EMPTY_VALUE;
           cciDu[i] = EMPTY_VALUE;
           trend[i] = trend[i+1];
           slope[i] = slope[i+1];
         
             if (cci[i]>0)        trend[i]=  1; 
             if (cci[i]<0)        trend[i]= -1;
             if (cci[i]>cci[i+1]) slope[i]=  1; 
             if (cci[i]<cci[i+1]) slope[i]= -1; 
             if (trend[i] == 1) { if (slope[i]== 1) cciU[i] = cci[i]; else cciUd[i] = cci[i]; }
             if (trend[i] ==-1) { if (slope[i]==-1) cciD[i] = cci[i]; else cciDu[i] = cci[i]; }
         }
      }
      
      //
      //
      //
      //
      //
   
      if (alertsOn)
      {
        if (alertsOnCurrent)
             int whichBar = 0;
        else     whichBar = 1;
         
        if (trend[whichBar] != trend[whichBar+1])
        if (trend[whichBar] == 1)
              doAlert("uptrend");
        else  doAlert("downtrend");       
   }
   
   return(0);
}

//+------------------------------------------------------------------+
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," T3 cci ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," T3 cci "),message);
             if (alertsSound)   PlaySound(soundfile);
      }
}
 
//
//
//
//
//

double iT3(double price,int shift)
{
   int i = Bars-shift-1;
   if (i < 1)
      {
         emas[i][0] = price;
         emas[i][1] = price;
         emas[i][2] = price;
         emas[i][3] = price;
         emas[i][4] = price;
         emas[i][5] = price;
      }
   else
      {
         emas[i][0] = emas[i-1][0]+alpha*(price     -emas[i-1][0]);
         emas[i][1] = emas[i-1][1]+alpha*(emas[i][0]-emas[i-1][1]);
         emas[i][2] = emas[i-1][2]+alpha*(emas[i][1]-emas[i-1][2]);
         emas[i][3] = emas[i-1][3]+alpha*(emas[i][2]-emas[i-1][3]);
         emas[i][4] = emas[i-1][4]+alpha*(emas[i][3]-emas[i-1][4]);
         emas[i][5] = emas[i-1][5]+alpha*(emas[i][4]-emas[i-1][5]);
      }
   return(c1*emas[i][5] + c2*emas[i][4] + c3*emas[i][3] + c4*emas[i][2]);
}


